"""
ssss
"""

import functools
import os
import warnings

import requests
from selenium import webdriver
from selenium.webdriver.firefox.options import Options


def deprecated(func):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.
    https://stackoverflow.com/questions/2536307/decorators-in-the-python-standard-lib-deprecated-specifically
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter('always', DeprecationWarning)  # turn off filter
        warnings.warn(
            f'Call to deprecated function {func.__name__}.',
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter('default', DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func


@deprecated
def create_driver(
    download_path,
    headless=False,
    adds_path=os.path.join('..', 'adds'),
    log_path=os.path.join('..', 'logs'),
):
    """

    :param download_path: Pasta de download
    :param headless: True or False
    :param adds_path: dddd
    :param log_path: dddsdsdsd
    :return:
    :rtype: object
    """
    print()

    # Create directory
    os.makedirs(adds_path, exist_ok=True)
    os.makedirs(log_path, exist_ok=True)
    os.makedirs(download_path, exist_ok=True)

    # Xpath
    url = 'https://addons.mozilla.org/firefox/downloads/file/3588871/xpath_finder-1.0.2-fx.xpi'
    r = requests.get(url)
    with open(os.path.join(adds_path, 'xpath.xpi'), 'wb') as f:
        f.write(r.content)

        # Driver Firefox com Profile
    profile = webdriver.FirefoxProfile()
    profile.set_preference('intl.accept_languages', 'pt-BR, pt')
    profile.set_preference('browser.download.folderList', 2)
    profile.set_preference('browser.download.dir', download_path)
    profile.set_preference('browser.download.manager.showWhenStarting', 'false')
    profile.set_preference(
        'browser.helperApps.neverAsk.saveToDisk',
        'application/octet-stream;application/vnd.ms-excel;text/html',
    )

    # profile.set_preference('browser.download.manager.showWhenStarting', False)
    # profile.set_preference('browser.helperApps.neverAsk.saveToDisk','application/zip,application/vnd.google-earth.kml+xml,application/rar,application/pdf,application/vnd.ms-excel,application/octet-stream,application/msword,text/xml,text/kml,application/pdf,application/vnd.adobe.xfdf,application/vnd.fdf,application/vnd.adobe.xdp+xml,image/x-png,image/png,image/jpeg,text/plain,text/html,application/x-msdownload')
    # profile.set_preference('browser.helperApps.alwaysAsk.force', False)
    profile.set_preference('browser.aboutConfig.showWarning', False)
    # profile.set_preference('pdfjs.disabled', True)
    # profile.set_preference('print.print_headerright', '')
    # profile.set_preference('print.print_headercenter', '')
    # profile.set_preference('print.print_headerleft', '')
    # profile.set_preference('print.print_footerright', '')
    # profile.set_preference('print.print_footercenter', '')
    # profile.set_preference('print.print_footerleft', '')

    profile.update_preferences()

    # Driver Firefox com Options
    options = Options()
    options.headless = headless

    # Driver
    global driver
    driver = webdriver.Firefox(
        firefox_profile=profile,
        options=options,
        service_log_path=os.path.join(log_path, 'geckodriver.log'),
    )
    driver.install_addon(
        os.path.abspath(os.path.join(adds_path, 'xpath.xpi')), temporary=True
    )
    return driver


if __name__ == '__main__':
    pass



"""
Equivalente ao Webdriver Project
Evita a necessidade de instalar drivers

https://github.com/mozilla/geckodriver/releases
out/22

Usei em:
- br_arisp
"""

import platform
import tarfile
import time
from pathlib import Path
from zipfile import ZipFile

import requests
from selenium import webdriver
from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.firefox.service import Service as FirefoxService
from .adds import add_extension_xpath


class Driver(webdriver.Firefox):
    """
    Cria driver customizado do Selenium

    :param webdriver: _description_
    :type webdriver: _type_
    """

    def __init__(self, my_driver_path, my_logs_path, *args, **kwargs):
        """

        verify_ssl

        :param my_driver_path: _description_
        :type my_driver_path: pathlib
        :param my_logs_path: _description_
        :type my_logs_path: pathlib
        """
        # Services
        gecko_path = get_path_geckodriver(
            my_driver_path, verify_ssl=kwargs['verify_ssl']
        )

        # Logs
        logs_filepath = my_logs_path / 'geckodriver.log'

        # Services
        my_service = FirefoxService(
            executable_path=gecko_path, log_path=logs_filepath
        )

        # Options
        my_options = FirefoxOptions()
        my_options.headless = False
        my_options.set_preference('intl.accept_languages', 'pt-BR, pt')

        # Driver
        my_driver = super(Driver, self)
        my_driver.__init__(service=my_service, options=my_options)

    def add_extension_xpath(self, my_adds_path):
        """
        Adiciona Xpath extension

        :param my_adds_path: Pasta da Extensão
        :type my_adds_path: pathlib
        """
        add_extension_xpath(self, my_adds_path)


def _check_geckodriver_exists(path):
    """
    _summary_

    :param path: _description_
    :type path: _type_
    :return: _description_
    :rtype: _type_
    """

    # Paths
    path.mkdir(exist_ok=True)
    gecko_win_filepath = path / 'geckodriver.exe'
    gecko_linux_filepath = path / 'geckodriver'

    if platform.system() == 'Windows':
        if gecko_win_filepath.is_file():
            return True

        elif not gecko_win_filepath.is_file():
            return False

    elif platform.system() == 'Linux':
        if gecko_linux_filepath.is_file():
            return True

        elif not gecko_linux_filepath.is_file():
            return False

    else:
        return None


def _get_geckodriver(path, verify_ssl=False):
    """
    Faz o download do geckodriver!
    TODO: Testar no windows!

    :param path:
    :return:
    """
    # Test
    has_geckodriver = _check_geckodriver_exists(path)

    # print(gecko_zip_filepath)
    if not has_geckodriver:
        if platform.system() == 'Windows':
            # Download do geckodriver
            url = 'https://github.com/mozilla/geckodriver/releases/download/v0.32.0/geckodriver-v0.32.0-win64.zip'
            r = requests.get(url, timeout=60, verify=verify_ssl)

            # Save
            gecko_zip_filepath = path / Path(url).name
            with open(gecko_zip_filepath, 'wb') as f:
                f.write(r.content)

            # Extract
            with ZipFile(gecko_zip_filepath, 'r') as zip_ref:
                zip_ref.extractall(path)

        elif platform.system() == 'Linux':
            # Download do geckodriver
            url = 'https://github.com/mozilla/geckodriver/releases/download/v0.32.0/geckodriver-v0.32.0-linux64.tar.gz'
            r = requests.get(url, timeout=60, verify=verify_ssl)

            # Save
            gecko_zip_filepath = path / Path(url).name
            with open(gecko_zip_filepath, 'wb') as f:
                f.write(r.content)

            # Extract
            with tarfile.open(gecko_zip_filepath, 'r') as tar_ref:
                tar_ref.extractall(path)

    elif has_geckodriver:
        print(f'Geckodriver already in {path}')


def get_path_geckodriver(path, verify_ssl=False):
    """
    _summary_

    :param path: _description_
    :type path: _type_
    :return: _description_
    :rtype: _type_
    """
    # Faz download se for necessário
    _get_geckodriver(path, verify_ssl=verify_ssl)

    # Path
    if platform.system() == 'Windows':
        _gecko_path = path / 'geckodriver.exe'
        _gecko_path = _gecko_path.resolve().as_posix().replace('/', '\\')
        return _gecko_path

    elif platform.system() == 'Linux':
        _gecko_path = path / 'geckodriver'
        return _gecko_path

    else:
        print(f'Ajustar para plataforma {platform.system()}')
        return None


if __name__ == '__main__':
    # Imports
    import time
    from paths import driver_path, logs_path

    # Services
    gecko_path = get_path_geckodriver(driver_path)

    # Logs
    logs_filepath = logs_path / 'geckodriver.log'

    # Services
    service = FirefoxService(executable_path=gecko_path, log_path=logs_filepath)

    # Options
    options = FirefoxOptions()
    options.headless = False
    options.set_preference('intl.accept_languages', 'pt-BR, pt')

    # Driver
    driver = webdriver.Firefox(service=service, options=options)
    driver.get('https://github.com/SergeyPirogov/webdriver_manager')
    driver.maximize_window()

    # Close Connection
    time.sleep(4)
    driver.quit()

    # # Options
    # from selenium.webdriver.chrome.service import Service as ChromeService
    # # https://chromedriver.storage.googleapis.com/index.html?path=107.0.5304.62/
    # options = webdriver.ChromeOptions()
    # #options.add_experimental_option("excludeSwitches", ["enable-automation"])
    # #options.add_experimental_option("useAutomationExtension", False)

    # # ddd
    # service = ChromeService(
    #     # executable_path=gecko_path
    # )
    # driver = webdriver.Chrome(service=service, options=options)
    # driver.get('https://github.com/SergeyPirogov/webdriver_manager')

    # # Close Connection
    # time.sleep(4)
    # driver.quit()

    # Message
    print('Driver close!!')



"""
Pastas Projeto
"""


from pathlib import Path

project_path = Path(__file__).parents[2]

# Scrapy
scrapy_path = project_path / 'scrapy'
scrapy_path.mkdir(exist_ok=True)

driver_path = scrapy_path / 'driver'
driver_path.mkdir(exist_ok=True)

logs_path = scrapy_path / 'logs'
logs_path.mkdir(exist_ok=True)

adds_path = scrapy_path / 'adds'
adds_path.mkdir(exist_ok=True)


if __name__ == '__main__':
    print(project_path)



"""
_summary_

:return: _description_
:rtype: _type_
"""


import requests


def add_extension_xpath(my_driver, adds_path):
    """
    _summary_

    :param my_driver: _description_
    :type my_driver: _type_
    """

    # Add-ons Xpath
    xpath_path = adds_path / 'xpath.xpi'
    xpath_path = xpath_path.absolute().resolve()
    if not xpath_path.is_file():
        url = 'https://addons.mozilla.org/firefox/downloads/file/3588871/xpath_finder-1.0.2-fx.xpi'
        r = requests.get(url, timeout=60, verify=False)

        with open(xpath_path, 'wb') as f:
            f.write(r.content)

    # Add
    my_driver.install_addon(str(xpath_path), temporary=True)
    return my_driver


if __name__ == '__main__':
    pass



"""
ssss
"""

import errno
import os
import stat

import chardet


def predict_encoding(file_path, n_lines=30):
    """
    Predict Encoding determines the encoding of a given file,
    so that when reading the file (via 'Pandas', for example)
    characters, accents and symbols are spelled correctly.

    # Examples to Use
    file_encoding = predict_encoding('data.csv')
    print(file_encoding)
    df = pd.read_csv('data.csv', encoding=file_encoding)

    :param file_path: Name and path to file
    :param n_lines: Number of lines to read to set encoding. Parameter Optional.
    Default: 30 lines.
    :return: Encoding.
    """

    # Open the file as binary data
    with open(file_path, 'rb') as f:
        # Join binary lines for specified number of lines
        raw_data = b''.join([f.readline() for line in range(n_lines)])
    enc = chardet.detect(raw_data)['encoding']
    return enc


def handle_remove_readonly(func, path, exc):
    """
    dddd

    Example:
    shutil.rmtree(
        input_path,
        ignore_errors=False,
        onerror=handle_remove_readonly
    )

    References:
    https://stackoverflow.com/questions/1213706/what-user-do-python-scripts-run-as-in-windows

    :param func: _description_
    :type func: _type_
    :param path: _description_
    :type path: _type_
    :param exc: _description_
    :type exc: _type_
    """

    excvalue = exc[1]
    if func in (os.rmdir, os.remove) and excvalue.errno == errno.EACCES:
        os.chmod(path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)  # 0777
        func(path)
    else:
        raise


if __name__ == '__main__':
    pass



"""
Ajuste
"""


# Inteiro para Romano
def int_to_roman(num):
    """

    https://www.w3resource.com/python-exercises/class-exercises/python-class-exercise-1.php
    :param num:
    :return:
    """
    lookup = [
        (1000, 'M'),
        (900, 'CM'),
        (500, 'D'),
        (400, 'CD'),
        (100, 'C'),
        (90, 'XC'),
        (50, 'L'),
        (40, 'XL'),
        (10, 'X'),
        (9, 'IX'),
        (5, 'V'),
        (4, 'IV'),
        (1, 'I'),
    ]
    res = ''
    for (n, roman) in lookup:
        (d, num) = divmod(num, n)
        res += roman * d
    return res


# Romano para Inteiro
def roman_to_int(s):
    """
    https://www.tutorialspoint.com/roman-to-integer-in-python
    :type s: str
    :rtype: int
    """
    roman = {
        'I': 1,
        'V': 5,
        'X': 10,
        'L': 50,
        'C': 100,
        'D': 500,
        'M': 1000,
        'IV': 4,
        'IX': 9,
        'XL': 40,
        'XC': 90,
        'CD': 400,
        'CM': 900
    }

    i = 0
    num = 0
    while i < len(s):
        if i + 1 < len(s) and s[i:i + 2] in roman:
            num += roman[s[i:i + 2]]
            i += 2
        else:
            # print(i)
            num += roman[s[i]]
            i += 1
    return num


if __name__ == '__main__':
    # Para testes
    print(int_to_roman(1))
    print(int_to_roman(156))
    print(roman_to_int('III'))
    print(roman_to_int('CDXLIII'))
